"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var checkTypes = require("check-types");
var traverson = require("traverson-promise");
var JsonHalAdapter = require("traverson-hal");
var q = require("q");
var logger_1 = require("./logger");
traverson.registerMediaType(JsonHalAdapter.mediaType, JsonHalAdapter);
var pactURLPattern = "/pacts/provider/%s/latest";
var pactURLPatternWithTag = "/pacts/provider/%s/latest/%s";
var Broker = (function () {
    function Broker(options) {
        this.options = options;
        this.__requestOptions = this.options.username && this.options.password ? {
            "auth": {
                "user": this.options.username,
                "password": this.options.password
            }
        } : {};
    }
    Broker.create = function (options) {
        options.username = options.username;
        options.password = options.password;
        options.tags = options.tags || [];
        checkTypes.assert.nonEmptyString(options.brokerUrl);
        checkTypes.assert.nonEmptyString(options.provider);
        if (options.tags) {
            checkTypes.assert.array.of.string(options.tags);
        }
        if (options.username) {
            checkTypes.assert.string(options.username);
        }
        if (options.password) {
            checkTypes.assert.string(options.password);
        }
        return new Broker(options);
    };
    Broker.prototype.findPacts = function (tag) {
        logger_1.default.debug("finding pacts for Provider:", this.options.provider, ", Tag:", tag);
        var linkName = tag ? "pb:latest-provider-pacts-with-tag" : "pb:latest-provider-pacts";
        return traverson
            .from(this.options.brokerUrl)
            .withTemplateParameters({ provider: this.options.provider, tag: tag })
            .withRequestOptions(this.__requestOptions)
            .jsonHal()
            .follow(linkName)
            .getResource()
            .result;
    };
    Broker.prototype.findConsumers = function () {
        var _this = this;
        logger_1.default.debug("Finding consumers");
        var promises = (this.options.tags.length > 0) ? this.options.tags.map(this.findPacts, this) : [this.findPacts()];
        return q.all(promises)
            .then(function (values) {
            var pactUrls = {};
            values.forEach(function (response) {
                if (response && response._links && response._links.pacts) {
                    response._links.pacts.forEach(function (pact) { return pactUrls[pact.title] = pact.href; });
                }
            });
            return Object.keys(pactUrls).reduce(function (pacts, key) {
                pacts.push(pactUrls[key]);
                return pacts;
            }, []);
        })
            .catch(function () { return q.reject("Unable to find pacts for given provider '" + _this.options.provider + "' and tags '" + _this.options.tags + "'"); });
    };
    return Broker;
}());
exports.Broker = Broker;
exports.default = Broker.create;
//# sourceMappingURL=broker.js.map